import pytest
from unittest.mock import patch, MagicMock
import ctypes
from everytools.everytools import EveryTools, SortType
from everytools.constants import RequestFlag
from everytools.constants import SortType as SortTypeConst


@pytest.fixture
def mock_everytools():
    """Fixture to create a mocked EveryTools instance."""
    with patch("ctypes.WinDLL") as mock_windll:
        # Mock the DLL object
        mock_dll = MagicMock()
        mock_windll.return_value = mock_dll

        # Mock the version functions called in __init__
        mock_dll.Everything_GetMajorVersion.return_value = 1
        mock_dll.Everything_GetMinorVersion.return_value = 4
        mock_dll.Everything_GetRevision.return_value = 1
        mock_dll.Everything_GetBuildNumber.return_value = 1020

        # Create an instance of EveryTools, which will now use the mock_dll
        et = EveryTools(machine=64)
        # Attach the mock_dll to the instance for later inspection in tests
        et.mock_dll = mock_dll
        return et


def test_get_search_keyword(mock_everytools):
    """测试get_search_keyword方法"""
    et = mock_everytools
    et.mock_dll.Everything_GetSearchW.return_value = "test_keyword"

    result = et.get_search_keyword()

    et.mock_dll.Everything_GetSearchW.assert_called_once()
    assert result == "test_keyword"


def test_get_num_total_results(mock_everytools):
    """测试获取结果数量的方法"""
    et = mock_everytools
    et.num_total_results = 100
    et.num_total_file = 80
    et.num_total_folder = 20

    assert et.get_num_total_results() == 100
    assert et.get_num_total_file() == 80
    assert et.get_num_total_folder() == 20


def test_results(mock_everytools):
    """测试results方法"""
    et = mock_everytools

    # 设置mock返回值
    et.mock_dll.Everything_GetNumResults.return_value = 2

    # 模拟返回结果
    et.mock_dll.Everything_GetResultFileNameW.side_effect = ["file1.txt", "file2.txt"]
    et.mock_dll.Everything_GetResultPathW.side_effect = ["C:\\path1", "C:\\path2"]
    et.mock_dll.Everything_GetResultExtensionW.side_effect = ["txt", "txt"]
    et.mock_dll.Everything_IsFileResult.side_effect = [1, 1]
    et.mock_dll.Everything_IsFolderResult.side_effect = [0, 0]
    et.mock_dll.Everything_IsVolumeResult.side_effect = [0, 0]

    # 创建一个模拟的filetime和size结果
    filetime_value = 116444736000000000
    size_value = 1024

    # 注意：GetResultDateCreated和GetResultDateModified将修改指向的值
    def mock_get_result_date(index, pointer):
        # 直接修改指针值
        pointer.value = filetime_value
        return True

    def mock_get_result_size(index, pointer):
        # 直接修改指针值
        pointer.value = size_value
        return True

    # 使用side_effect模拟修改指针
    et.mock_dll.Everything_GetResultDateModified.side_effect = mock_get_result_date
    et.mock_dll.Everything_GetResultDateCreated.side_effect = mock_get_result_date
    et.mock_dll.Everything_GetResultSize.side_effect = mock_get_result_size

    # 确保get_num_total_results返回正确的值
    et.get_num_total_results = MagicMock(return_value=2)

    # 执行测试 - results返回的是生成器，需要转为列表
    results = list(et.results(max_num=2, sort_type=SortTypeConst.NAME_ASCENDING.value))

    # 验证结果
    assert len(results) == 2
    assert results[0]["name"] == "file1.txt"
    assert results[0]["path"] == "C:\\path1"
    assert results[0]["file_extension"] == "txt"
    assert results[0]["is_file"] == 1
    assert results[0]["is_folder"] == 0
    assert "size" in results[0]
    assert "created_date" in results[0]
    assert "modified_date" in results[0]


def test_set_sort(mock_everytools):
    """测试设置排序方法"""
    et = mock_everytools
    et.set_sort(SortTypeConst.PATH_ASCENDING.value)
    et.mock_dll.Everything_SetSort.assert_called_with(
        SortTypeConst.PATH_ASCENDING.value
    )


def test_get_last_error(mock_everytools):
    """测试获取最后错误方法"""
    et = mock_everytools
    et.mock_dll.Everything_GetLastError.return_value = 1

    result = et.get_last_error()

    et.mock_dll.Everything_GetLastError.assert_called_once()
    assert result == 1


def test_sort_results_by_path(mock_everytools):
    """测试按路径排序结果方法"""
    et = mock_everytools
    et.sort_results_by_path()
    # 正确的函数是Everything_SortResultsByPath
    et.mock_dll.Everything_SortResultsByPath.assert_called_once()


def test_is_db_loaded(mock_everytools):
    """测试数据库加载状态方法"""
    et = mock_everytools
    et.mock_dll.Everything_IsDBLoaded.return_value = 1

    result = et.is_db_loaded()

    et.mock_dll.Everything_IsDBLoaded.assert_called_once()
    assert result is True


def test_is_admin(mock_everytools):
    """测试管理员权限检查方法"""
    et = mock_everytools
    et.mock_dll.Everything_IsAdmin.return_value = 0

    result = et.is_admin()

    et.mock_dll.Everything_IsAdmin.assert_called_once()
    assert result is False


def test_exit(mock_everytools):
    """测试退出方法"""
    et = mock_everytools
    et.exit()
    # 正确的函数是Everything_Exit
    et.mock_dll.Everything_Exit.assert_called_once()
