#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
测试Everything现代接口模块
"""

import pytest
from unittest.mock import Mock, patch, MagicMock
from everytools.core.everything import Everything


@pytest.fixture
def mock_everytools():
    """模拟EveryTools实例"""
    with patch("everytools.core.everything.EveryTools") as mock_et:
        mock_instance = Mock()
        mock_et.return_value = mock_instance
        yield mock_instance


def test_everything_init(mock_everytools):
    """测试Everything类初始化"""
    everything = Everything(machine=64)

    # 验证初始化参数
    assert everything._query == ""
    assert everything._match_case is False
    assert everything._match_path is False
    assert everything._match_whole_word is False
    assert everything._regex is False
    assert everything._max_results is None
    assert everything._sort_type == 1


def test_everything_init_default_machine(mock_everytools):
    """测试Everything类默认机器架构初始化"""
    everything = Everything()

    # 验证默认参数
    assert everything._query == ""
    assert everything._match_case is False


def test_search_method(mock_everytools):
    """测试search方法"""
    everything = Everything()
    result = everything.search("test query")

    # 验证链式调用
    assert result is everything
    assert everything._query == "test query"


def test_match_case_method(mock_everytools):
    """测试match_case方法"""
    everything = Everything()

    # 测试启用区分大小写
    result = everything.match_case(True)
    assert result is everything
    assert everything._match_case is True

    # 测试禁用区分大小写
    result = everything.match_case(False)
    assert result is everything
    assert everything._match_case is False

    # 测试默认参数
    result = everything.match_case()
    assert result is everything
    assert everything._match_case is True


def test_match_path_method(mock_everytools):
    """测试match_path方法"""
    everything = Everything()

    # 测试启用路径匹配
    result = everything.match_path(True)
    assert result is everything
    assert everything._match_path is True

    # 测试禁用路径匹配
    result = everything.match_path(False)
    assert result is everything
    assert everything._match_path is False

    # 测试默认参数
    result = everything.match_path()
    assert result is everything
    assert everything._match_path is True


def test_match_whole_word_method(mock_everytools):
    """测试match_whole_word方法"""
    everything = Everything()

    # 测试启用全字匹配
    result = everything.match_whole_word(True)
    assert result is everything
    assert everything._match_whole_word is True

    # 测试禁用全字匹配
    result = everything.match_whole_word(False)
    assert result is everything
    assert everything._match_whole_word is False

    # 测试默认参数
    result = everything.match_whole_word()
    assert result is everything
    assert everything._match_whole_word is True


def test_use_regex_method(mock_everytools):
    """测试use_regex方法"""
    everything = Everything()

    # 测试启用正则表达式
    result = everything.use_regex(True)
    assert result is everything
    assert everything._regex is True

    # 测试禁用正则表达式
    result = everything.use_regex(False)
    assert result is everything
    assert everything._regex is False

    # 测试默认参数
    result = everything.use_regex()
    assert result is everything
    assert everything._regex is True


def test_sort_by_method(mock_everytools):
    """测试sort_by方法"""
    everything = Everything()

    result = everything.sort_by(5)
    assert result is everything
    assert everything._sort_type == 5


def test_limit_method(mock_everytools):
    """测试limit方法"""
    everything = Everything()

    result = everything.limit(100)
    assert result is everything
    assert everything._max_results == 100


def test_results_method(mock_everytools):
    """测试results方法"""
    everything = Everything()
    everything._query = "test"
    everything._match_path = True
    everything._match_case = True
    everything._match_whole_word = True
    everything._regex = True
    everything._max_results = 50
    everything._sort_type = 3

    # 模拟返回结果
    mock_results = Mock()
    mock_everytools.results.return_value = mock_results

    result = everything.results()

    # 验证调用参数
    mock_everytools.search.assert_called_once_with(
        "test",
        math_path=True,
        math_case=True,
        whole_world=True,
        regex=True,
    )
    mock_everytools.results.assert_called_once_with(max_num=50, sort_type=3)

    assert result is mock_results


def test_context_manager(mock_everytools):
    """测试上下文管理器"""
    with Everything() as everything:
        assert isinstance(everything, Everything)
        result = everything.search("test")
        assert result is everything


def test_method_chaining(mock_everytools):
    """测试方法链式调用"""
    everything = Everything()

    result = (
        everything.search("test query")
        .match_case(True)
        .match_path(True)
        .match_whole_word(False)
        .use_regex(True)
        .sort_by(2)
        .limit(100)
    )

    assert result is everything
    assert everything._query == "test query"
    assert everything._match_case is True
    assert everything._match_path is True
    assert everything._match_whole_word is False
    assert everything._regex is True
    assert everything._sort_type == 2
    assert everything._max_results == 100
