import struct
import time
import pytest
from unittest.mock import patch, MagicMock
from everytools.everytools import get_time, EveryTools


def test_get_time():
    """
    Tests the get_time function for converting Windows FILETIME to a datetime string.
    """
    # This FILETIME value corresponds to the Unix epoch (1970-01-01 00:00:00 UTC).
    # The get_time function should convert it to the local time representation of the epoch.
    epoch_winticks = 116444736000000000
    epoch_filetime_bytes = struct.pack("<Q", epoch_winticks)

    # The expected result is the local time representation of 0 seconds since epoch.
    expected_epoch_str = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(0))

    assert get_time(epoch_filetime_bytes) == expected_epoch_str

    # This FILETIME value corresponds to one hour after the Unix epoch.
    one_hour_after_epoch_winticks = 116444736000000000 + (3600 * 10**7)
    one_hour_filetime_bytes = struct.pack("<Q", one_hour_after_epoch_winticks)

    # The expected result is the local time representation of 3600 seconds since epoch.
    expected_one_hour_str = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(3600))

    assert get_time(one_hour_filetime_bytes) == expected_one_hour_str


@pytest.fixture
def mock_everytools():
    """Fixture to create a mocked EveryTools instance."""
    with patch("ctypes.WinDLL") as mock_windll:
        # Mock the DLL object
        mock_dll = MagicMock()
        mock_windll.return_value = mock_dll

        # Mock the version functions called in __init__
        mock_dll.Everything_GetMajorVersion.return_value = 1
        mock_dll.Everything_GetMinorVersion.return_value = 4
        mock_dll.Everything_GetRevision.return_value = 1
        mock_dll.Everything_GetBuildNumber.return_value = 1020

        # Create an instance of EveryTools, which will now use the mock_dll
        et = EveryTools(machine=64)
        # Attach the mock_dll to the instance for later inspection in tests
        et.mock_dll = mock_dll
        return et


def test_everytools_init(mock_everytools):
    """Test that EveryTools initializes correctly with the mocked DLL."""
    et = mock_everytools
    assert et.version == "1.4.1.1020"
    et.mock_dll.Everything_GetMajorVersion.assert_called_once()


def test_search_calls(mock_everytools):
    """Test the main search method and its parameter flags."""
    et = mock_everytools
    et.search("test.txt", math_path=True, math_case=True, whole_world=True, regex=True)

    et.mock_dll.Everything_SetSearchW.assert_called_with("test.txt")
    et.mock_dll.Everything_SetMatchPath.assert_called_with(True)
    et.mock_dll.Everything_SetMatchCase.assert_called_with(True)
    et.mock_dll.Everything_SetMatchWholeWord.assert_called_with(True)
    et.mock_dll.Everything_SetRegex.assert_called_with(True)
    et.mock_dll.Everything_QueryW.assert_called_with(True)


@pytest.mark.parametrize(
    "search_func, helper_keywords, expected_search_str",
    [
        (
            "search_audio",
            "song",
            "ext:aac;ac3;aif;aifc;aiff;au;cda;dts;fla;flac;it;m1a;m2a;m3u;m4a;mid;midi;mka;mod;mp2;mp3;mpa;ogg;ra;rmi;spc;rmi;snd;umx;voc;wav;wma;xm song",
        ),
        (
            "search_zip",
            "archive",
            "ext:7z;ace;arj;bz2;cab;gz;gzip;jar;r00;r01;r02;r03;r04;r05;r06;r07;r08;r09;r10;r11;r12;r13;r14;r15;r16;r17;r18;r19;r20;r21;r22;r23;r24;r25;r26;r27;r28;r29;rar;tar;tgz;z;zip archive",
        ),
        (
            "search_doc",
            "report",
            "ext:c;chm;cpp;csv;cxx;doc;docm;docx;dot;dotm;dotx;h;hpp;htm;html;hxx;ini;java;lua;mht;mhtml;odt;pdf;potx;potm;ppam;ppsm;ppsx;pps;ppt;pptm;pptx;rtf;sldm;sldx;thmx;txt;vsd;wpd;wps;wri;xlam;xls;xlsb;xlsm;xlsx;xltm;xltx;xml report",
        ),
        ("search_exe", "setup", "ext:bat;cmd;exe;msi;msp;scr setup"),
        ("search_folder", "my_folder", "folder: my_folder"),
        (
            "search_pic",
            "photo",
            "ext:ani;bmp;gif;ico;jpe;jpeg;jpg;pcx;png;psd;tga;tif;tiff;webp;wmf photo",
        ),
        (
            "search_video",
            "movie",
            "ext:3g2;3gp;3gp2;3gpp;amr;amv;asf;avi;bdmv;bik;d2v;divx;drc;dsa;dsm;dss;dsv;evo;f4v;flc;fli;flic;flv;hdmov;ifo;ivf;m1v;m2p;m2t;m2ts;m2v;m4b;m4p;m4v;mkv;mp2v;mp4;mp4v;mpe;mpeg;mpg;mpls;mpv2;mpv4;mov;mts;ogm;ogv;pss;pva;qt;ram;ratdvd;rm;rmm;rmvb;roq;rpm;smil;smk;swf;tp;tpr;ts;vob;vp6;webm;wm;wmp;wmv movie",
        ),
    ],
)
def test_search_helpers(
    mock_everytools, search_func, helper_keywords, expected_search_str
):
    """Test various search helper methods to ensure they call search() with the correct query."""
    et = mock_everytools
    method_to_call = getattr(et, search_func)
    method_to_call(helper_keywords)
    et.mock_dll.Everything_SetSearchW.assert_called_with(expected_search_str)


def test_search_ext(mock_everytools):
    """Test search_ext method."""
    et = mock_everytools
    et.search_ext("py", "test")
    et.mock_dll.Everything_SetSearchW.assert_called_with("ext:py test")


def test_search_in_located(mock_everytools):
    """Test search_in_located method."""
    et = mock_everytools
    et.search_in_located("C:\\Users", "document")
    et.mock_dll.Everything_SetSearchW.assert_called_with("C:\\Users document")
