import pytest
from unittest.mock import patch, MagicMock
import ctypes
from everytools.everytools import EveryTools
from everytools.exceptions import (
    EverythingError,
    MemoryError,
    IPCError,
    RegisterClassError,
    CreateWindowError,
    CreateThreadError,
    InvalidIndexError,
    InvalidCallError,
    raise_for_error_code,
)
from everytools.constants import ErrorCode


def test_raise_for_error_code_no_error():
    """测试正常情况下不会抛出异常"""
    # 不应该抛出任何异常
    raise_for_error_code(ErrorCode.EVERYTHING_OK)


def test_raise_for_error_code_memory_error():
    """测试内存错误情况"""
    with pytest.raises(MemoryError):
        raise_for_error_code(ErrorCode.EVERYTHING_ERROR_MEMORY)


def test_raise_for_error_code_ipc_error():
    """测试IPC错误情况"""
    with pytest.raises(IPCError):
        raise_for_error_code(ErrorCode.EVERYTHING_ERROR_IPC)


def test_raise_for_error_code_registerclass_error():
    """测试注册类错误情况"""
    with pytest.raises(RegisterClassError):
        raise_for_error_code(ErrorCode.EVERYTHING_ERROR_REGISTERCLASSEX)


def test_raise_for_error_code_createwindow_error():
    """测试创建窗口错误情况"""
    with pytest.raises(CreateWindowError):
        raise_for_error_code(ErrorCode.EVERYTHING_ERROR_CREATEWINDOW)


def test_raise_for_error_code_createthread_error():
    """测试创建线程错误情况"""
    with pytest.raises(CreateThreadError):
        raise_for_error_code(ErrorCode.EVERYTHING_ERROR_CREATETHREAD)


def test_raise_for_error_code_invalidindex_error():
    """测试无效索引错误情况"""
    with pytest.raises(InvalidIndexError):
        raise_for_error_code(ErrorCode.EVERYTHING_ERROR_INVALIDINDEX)


def test_raise_for_error_code_invalidcall_error():
    """测试无效调用错误情况"""
    with pytest.raises(InvalidCallError):
        raise_for_error_code(ErrorCode.EVERYTHING_ERROR_INVALIDCALL)


def test_raise_for_error_code_unknown_error():
    """测试未知错误情况"""
    # 使用一个不在映射中的错误码
    unknown_error_code = 999
    with pytest.raises(EverythingError):
        raise_for_error_code(unknown_error_code)


@pytest.fixture
def mock_everytools():
    """创建一个模拟的EveryTools实例"""
    with patch("ctypes.WinDLL") as mock_windll:
        mock_dll = MagicMock()
        mock_windll.return_value = mock_dll

        # 模拟版本函数
        mock_dll.Everything_GetMajorVersion.return_value = 1
        mock_dll.Everything_GetMinorVersion.return_value = 4
        mock_dll.Everything_GetRevision.return_value = 1
        mock_dll.Everything_GetBuildNumber.return_value = 1020

        et = EveryTools(machine=64)
        et.mock_dll = mock_dll
        return et


def test_error_handling_in_search(mock_everytools):
    """测试在搜索时检查错误

    注意：当前的EveryTools.search方法实际上没有在失败时抛出异常，
    我们测试该方法会调用Everything_QueryW和获取搜索结果数量
    """
    et = mock_everytools

    # 设置模拟的GetLastError返回
    et.mock_dll.Everything_GetLastError.return_value = ErrorCode.EVERYTHING_ERROR_MEMORY
    et.mock_dll.Everything_QueryW.return_value = 0  # 搜索失败

    # 搜索方法不会抛出异常，我们只测试它会调用正确的方法
    et.search("test_query")

    # 确保调用了正确的函数
    et.mock_dll.Everything_SetSearchW.assert_called_with("test_query")
    et.mock_dll.Everything_QueryW.assert_called_with(True)

    # 验证结果数量方法被调用
    et.mock_dll.Everything_GetNumResults.assert_called()
    et.mock_dll.Everything_GetTotFileResults.assert_called()
    et.mock_dll.Everything_GetTotFolderResults.assert_called()


def test_error_handling_in_results(mock_everytools):
    """测试在获取结果时检查错误情况

    注意：当前的EveryTools.results方法实际上没有在失败时抛出异常
    """
    et = mock_everytools

    # 我们需要模拟get_time函数的行为，避免实际调用time.localtime
    with patch("everytools.everytools.get_time", return_value="2023-01-01 00:00:00"):
        # 设置模拟的GetNumResults返回
        et.mock_dll.Everything_GetNumResults.return_value = 1

        # 模拟必要的返回值，以避免在results中遇到未定义的行为
        et.mock_dll.Everything_GetResultFileNameW.side_effect = ["test.txt"]
        et.mock_dll.Everything_GetResultPathW.side_effect = ["C:\\test"]
        et.mock_dll.Everything_GetResultExtensionW.side_effect = ["txt"]
        et.mock_dll.Everything_IsFileResult.side_effect = [1]
        et.mock_dll.Everything_IsFolderResult.side_effect = [0]
        et.mock_dll.Everything_IsVolumeResult.side_effect = [0]

        # 模拟size和date的获取
        def mock_get_value(index, pointer):
            pointer.value = 123
            return True

        et.mock_dll.Everything_GetResultDateCreated.side_effect = mock_get_value
        et.mock_dll.Everything_GetResultDateModified.side_effect = mock_get_value
        et.mock_dll.Everything_GetResultSize.side_effect = mock_get_value

        # 模拟调用实际的方法
        et.get_num_total_results = MagicMock(return_value=1)

        # 测试获取结果的行为
        results = list(et.results(max_num=1))

        # 验证调用了设置请求标志和查询的方法
        et.mock_dll.Everything_SetRequestFlags.assert_called()
        et.mock_dll.Everything_QueryW.assert_called_with(True)

        # 验证返回结果的基本结构
        assert len(results) == 1
        assert results[0]["name"] == "test.txt"
        assert results[0]["path"] == "C:\\test"
