#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
测试过滤器模块
Test filters module
"""

import pytest
from everytools.query.filters import FileFilter, FolderFilter, SizeFilter, DateFilter


class TestFileFilter:
    """测试文件过滤器"""

    def test_init(self):
        """测试初始化"""
        filter_obj = FileFilter()
        assert filter_obj._extensions == set()
        assert filter_obj._min_size is None
        assert filter_obj._max_size is None
        assert filter_obj._content is None
        assert filter_obj._duplicates is False

    def test_with_extensions(self):
        """测试扩展名过滤"""
        filter_obj = FileFilter()
        result = filter_obj.with_extensions("txt", ".py", "jpg")
        assert result is filter_obj  # 链式调用
        assert filter_obj._extensions == {"txt", "py", "jpg"}

    def test_with_size_range(self):
        """测试大小范围过滤"""
        filter_obj = FileFilter()
        result = filter_obj.with_size_range(min_size=1024, max_size=2048)
        assert result is filter_obj
        assert filter_obj._min_size == 1024
        assert filter_obj._max_size == 2048

    def test_with_content(self):
        """测试内容过滤"""
        filter_obj = FileFilter()
        result = filter_obj.with_content("test content")
        assert result is filter_obj
        assert filter_obj._content == "test content"

    def test_duplicates_only(self):
        """测试重复文件过滤"""
        filter_obj = FileFilter()
        result = filter_obj.duplicates_only()
        assert result is filter_obj
        assert filter_obj._duplicates is True

    def test_to_query_string_extensions(self):
        """测试扩展名查询字符串生成"""
        filter_obj = FileFilter().with_extensions("txt", "py")
        query = filter_obj.to_query_string()
        assert "ext:txt" in query or "ext:py" in query

    def test_to_query_string_size(self):
        """测试大小查询字符串生成"""
        filter_obj = FileFilter().with_size_range(min_size=1024, max_size=2048)
        query = filter_obj.to_query_string()
        assert "size:1024..2048" in query

    def test_to_query_string_content(self):
        """测试内容查询字符串生成"""
        filter_obj = FileFilter().with_content("test")
        query = filter_obj.to_query_string()
        assert 'content:"test"' in query

    def test_to_query_string_duplicates(self):
        """测试重复文件查询字符串生成"""
        filter_obj = FileFilter().duplicates_only()
        query = filter_obj.to_query_string()
        assert "dupe" in query


class TestSizeFilter:
    """测试大小过滤器"""

    def test_exactly(self):
        """测试精确大小"""
        filter_obj = SizeFilter().between(1024, 1024)
        query = filter_obj.to_query_string()
        assert "size:1024..1024" in query

    def test_greater_than(self):
        """测试大于"""
        filter_obj = SizeFilter().larger_than(1024)
        query = filter_obj.to_query_string()
        assert "size:>1024" in query

    def test_less_than(self):
        """测试小于"""
        filter_obj = SizeFilter().smaller_than(1024)
        query = filter_obj.to_query_string()
        assert "size:<1024" in query

    def test_between(self):
        """测试范围"""
        filter_obj = SizeFilter().between(1024, 2048)
        query = filter_obj.to_query_string()
        assert "size:1024..2048" in query


class TestDateFilter:
    """测试日期过滤器"""

    def test_today(self):
        """测试今天"""
        filter_obj = DateFilter.today()
        query = filter_obj.to_query_string()
        assert "today" in query

    def test_yesterday(self):
        """测试昨天"""
        filter_obj = DateFilter.yesterday()
        query = filter_obj.to_query_string()
        assert "yesterday" in query

    def test_this_week(self):
        """测试本周"""
        filter_obj = DateFilter.this_week()
        query = filter_obj.to_query_string()
        assert "thisweek" in query

    def test_last_week(self):
        """测试上周"""
        filter_obj = DateFilter.last_week()
        query = filter_obj.to_query_string()
        assert "lastweek" in query
