import os
import pytest
from unittest.mock import patch, mock_open, MagicMock
from everytools.everytools import download_dll


@pytest.fixture
def mock_os_makedirs():
    """模拟os.makedirs函数"""
    with patch("os.makedirs") as mock:
        yield mock


@pytest.fixture
def mock_requests_get():
    """模拟requests.get函数"""
    with patch("requests.get") as mock:
        mock_response = MagicMock()
        mock_response.content = b"mock dll content"
        mock.return_value = mock_response
        yield mock


@pytest.fixture
def mock_zipfile():
    """模拟zipfile.ZipFile"""
    with patch("zipfile.ZipFile") as mock:
        mock_zipfile_instance = MagicMock()
        mock_zipfile_instance.namelist.return_value = [
            "Everything64.dll",
            "Everything32.dll",
            "README.txt",
        ]

        # 模拟read方法返回不同的dll内容
        def mock_read(filename):
            if filename == "Everything64.dll":
                return b"mock 64bit dll content"
            elif filename == "Everything32.dll":
                return b"mock 32bit dll content"
            return b""

        mock_zipfile_instance.read.side_effect = mock_read
        mock.return_value = mock_zipfile_instance
        yield mock


@pytest.fixture
def mock_open_file():
    """模拟open函数"""
    with patch("builtins.open", mock_open()) as mock_file:
        yield mock_file


@pytest.fixture
def mock_os_remove():
    """模拟os.remove函数"""
    with patch("os.remove") as mock:
        yield mock


def test_download_dll(
    mock_os_makedirs, mock_requests_get, mock_zipfile, mock_open_file, mock_os_remove
):
    """测试下载DLL函数"""
    # 测试下载到自定义目录
    test_dll_dir = "test_dll_dir"
    download_dll(test_dll_dir)

    # 验证所有预期的函数调用
    mock_os_makedirs.assert_called_once_with(test_dll_dir, exist_ok=True)
    mock_requests_get.assert_called_once_with(
        "https://www.voidtools.com/Everything-SDK.zip"
    )
    mock_zipfile.assert_called_once()

    # 验证从zip文件提取的dll调用
    mock_open_calls = mock_open_file.mock_calls
    # 至少应该有3个open调用（一个用于保存zip文件，两个用于保存dll）
    assert len(mock_open_calls) >= 3

    # 验证删除临时zip文件
    mock_os_remove.assert_called_once_with("Everything-SDK.zip")


@pytest.mark.integration
def test_real_download_dll():
    """真实测试下载DLL函数 - 需要网络连接

    此测试标记为集成测试，默认不会运行
    使用 pytest -m integration 运行此测试
    """
    test_dll_dir = "test_real_dll_dir"

    try:
        # 实际执行下载函数
        download_dll(test_dll_dir)

        # 验证dll文件已下载
        assert os.path.exists(os.path.join(test_dll_dir, "Everything64.dll"))
        assert os.path.exists(os.path.join(test_dll_dir, "Everything32.dll"))
    finally:
        # 清理测试文件
        if os.path.exists(os.path.join(test_dll_dir, "Everything64.dll")):
            os.remove(os.path.join(test_dll_dir, "Everything64.dll"))
        if os.path.exists(os.path.join(test_dll_dir, "Everything32.dll")):
            os.remove(os.path.join(test_dll_dir, "Everything32.dll"))
        if os.path.exists(test_dll_dir):
            os.rmdir(test_dll_dir)
