#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
result模块的单元测试
"""

import os
import datetime
import struct
import pytest
from unittest.mock import MagicMock, patch
from everytools.core.result import FileResult, ResultSet


def test_file_result_init():
    """测试FileResult的初始化"""
    # 基础初始化
    result = FileResult(name="test.txt", path="C:\\temp")
    assert result.name == "test.txt"
    assert result.path == "C:\\temp"
    assert result.full_path == "C:\\temp\\test.txt"
    assert result.is_file == True
    assert result.is_folder == False
    assert result.is_volume == False

    # 测试full_path自动计算
    result = FileResult(name="test.txt", path="")
    assert result.full_path is None

    # 测试手动设置full_path
    result = FileResult(
        name="test.txt", path="C:\\temp", full_path="D:\\other\\test.txt"
    )
    assert result.full_path == "D:\\other\\test.txt"

    # 测试文件夹
    result = FileResult(name="folder", path="C:\\temp", is_file=False, is_folder=True)
    assert result.is_file == False
    assert result.is_folder == True


def test_file_result_to_dict():
    """测试FileResult的to_dict方法"""
    now = datetime.datetime.now()
    result = FileResult(
        name="test.txt",
        path="C:\\temp",
        size=1024,
        date_created=now,
        date_modified=now,
        date_accessed=now,
        date_run=now,
        extension="txt",
        attributes=0x20,  # 标准文件属性
        run_count=5,
        highlighted_name="<b>test</b>.txt",
        highlighted_path="C:\\<b>temp</b>",
    )

    result_dict = result.to_dict()
    assert result_dict["name"] == "test.txt"
    assert result_dict["path"] == "C:\\temp"
    assert result_dict["full_path"] == "C:\\temp\\test.txt"
    assert result_dict["size"] == 1024
    assert result_dict["date_created"] == now
    assert result_dict["date_modified"] == now
    assert result_dict["date_accessed"] == now
    assert result_dict["date_run"] == now
    assert result_dict["extension"] == "txt"
    assert result_dict["attributes"] == 0x20
    assert result_dict["is_file"] == True
    assert result_dict["is_folder"] == False
    assert result_dict["run_count"] == 5
    assert result_dict["highlighted_name"] == "<b>test</b>.txt"
    assert result_dict["highlighted_path"] == "C:\\<b>temp</b>"


def test_file_result_str_repr():
    """测试FileResult的字符串表示"""
    # 文件
    file_result = FileResult(name="test.txt", path="C:\\temp")
    assert str(file_result) == "C:\\temp\\test.txt "
    assert (
        repr(file_result)
        == "FileResult(name='test.txt', path='C:\\temp', is_file=True)"
    )

    # 文件夹
    folder_result = FileResult(
        name="folder", path="C:\\temp", is_file=False, is_folder=True
    )
    assert str(folder_result) == "C:\\temp\\folder (文件夹)"
    assert (
        repr(folder_result)
        == "FileResult(name='folder', path='C:\\temp', is_file=False)"
    )


@pytest.fixture
def mock_dll():
    """创建模拟的Everything DLL"""
    mock = MagicMock()

    # 模拟结果总数
    mock.Everything_GetNumResults.return_value = 3
    mock.Everything_GetTotFileResults.return_value = 2
    mock.Everything_GetTotFolderResults.return_value = 1

    # 模拟结果数据
    def get_result_filename(index):
        filenames = ["file1.txt", "file2.doc", "folder"]
        return filenames[index] if 0 <= index < len(filenames) else None

    def get_result_path(index):
        paths = ["C:\\temp", "C:\\temp", "C:\\"]
        return paths[index] if 0 <= index < len(paths) else None

    def is_file_result(index):
        return index < 2  # 前两个是文件

    def is_folder_result(index):
        return index == 2  # 最后一个是文件夹

    def is_volume_result(index):
        return False  # 没有卷结果

    def get_result_extension(index):
        extensions = ["txt", "doc", ""]
        return extensions[index] if 0 <= index < len(extensions) else None

    def get_result_size(index, buffer):
        sizes = [1024, 2048, 0]
        if 0 <= index < len(sizes):
            buffer.value = sizes[index]
        else:
            buffer.value = 0xFFFFFFFFFFFFFFFF  # 无效值

    def get_date_created(index, buffer):
        if index == 0:
            # Unix纪元时间之后的有效时间 (1970-01-02)
            buffer.value = 116444736000000000 + 864000000000  # 加一天
        elif index == 1:
            # 2000-01-01的Windows文件时间
            buffer.value = 125911584000000000
        else:
            buffer.value = 0xFFFFFFFFFFFFFFFF  # 无效值

    def get_result_attributes(index):
        attributes = [0x20, 0x20, 0x10]  # 0x20是标准文件，0x10是目录
        return attributes[index] if 0 <= index < len(attributes) else 0

    def get_run_count(index):
        run_counts = [5, 3, 0]
        return run_counts[index] if 0 <= index < len(run_counts) else 0

    mock.Everything_GetResultFileNameW.side_effect = get_result_filename
    mock.Everything_GetResultPathW.side_effect = get_result_path
    mock.Everything_IsFileResult.side_effect = is_file_result
    mock.Everything_IsFolderResult.side_effect = is_folder_result
    mock.Everything_IsVolumeResult.side_effect = is_volume_result
    mock.Everything_GetResultExtensionW.side_effect = get_result_extension
    mock.Everything_GetResultSize.side_effect = get_result_size
    mock.Everything_GetResultDateCreated.side_effect = get_date_created
    mock.Everything_GetResultDateModified.side_effect = (
        get_date_created  # 重用相同的日期逻辑
    )
    mock.Everything_GetResultDateAccessed.side_effect = (
        get_date_created  # 重用相同的日期逻辑
    )
    mock.Everything_GetResultDateRun.side_effect = (
        get_date_created  # 重用相同的日期逻辑
    )
    mock.Everything_GetResultAttributes.side_effect = get_result_attributes
    mock.Everything_GetResultRunCount.side_effect = get_run_count

    # 高亮相关
    mock.Everything_GetResultHighlightedFileNameW.return_value = None
    mock.Everything_GetResultHighlightedPathW.return_value = None

    return mock


def test_result_set_init(mock_dll):
    """测试ResultSet的初始化"""
    # 默认初始化
    result_set = ResultSet(mock_dll)
    assert result_set.total_results == 3
    assert result_set.total_files == 2
    assert result_set.total_folders == 1
    assert len(result_set) == 3

    # 限制最大结果数
    limited_result_set = ResultSet(mock_dll, max_results=1)
    assert limited_result_set.total_results == 3  # 总数不变
    assert len(limited_result_set) == 1  # 但迭代长度受限


def test_result_set_iteration(mock_dll):
    """测试ResultSet的迭代功能"""
    result_set = ResultSet(mock_dll)
    results = list(result_set)

    assert len(results) == 3

    # 验证第一个结果
    assert results[0].name == "file1.txt"
    assert results[0].path == "C:\\temp"
    assert results[0].is_file == True
    assert results[0].is_folder == False
    assert results[0].size == 1024

    # 验证第二个结果
    assert results[1].name == "file2.doc"
    assert results[1].extension == "doc"

    # 验证第三个结果（文件夹）
    assert results[2].name == "folder"
    assert results[2].is_file == False
    assert results[2].is_folder == True


def test_result_set_date_conversion(mock_dll):
    """测试ResultSet的日期转换功能"""
    # 转换为datetime
    datetime_result_set = ResultSet(mock_dll, convert_date=True)
    results = list(datetime_result_set)

    # 第一个结果的创建日期应该是Unix纪元时间
    assert isinstance(results[0].date_created, datetime.datetime)

    # 不转换（使用字符串）
    string_result_set = ResultSet(mock_dll, convert_date=False)
    results = list(string_result_set)

    # 结果的创建日期应该是格式化的字符串
    assert isinstance(results[0].date_created, str)


def test_result_set_to_list(mock_dll):
    """测试ResultSet的to_list方法"""
    result_set = ResultSet(mock_dll, max_results=2)
    result_list = result_set.to_list()

    assert len(result_list) == 2
    assert result_list[0]["name"] == "file1.txt"
    assert result_list[1]["name"] == "file2.doc"

    # 验证返回的是字典列表
    assert isinstance(result_list, list)
    assert all(isinstance(item, dict) for item in result_list)
