#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
测试搜索模块
Test search module
"""

import pytest
from unittest.mock import Mock, patch
from everytools.query.search import SearchBuilder
from everytools.query.filters import FileFilter
from everytools.constants import SortType, RequestFlag


class TestSearchBuilder:
    """测试搜索构建器"""

    def test_init(self):
        """测试初始化"""
        builder = SearchBuilder()
        assert builder._keywords == []
        assert builder._filters == []
        assert builder._match_case is False
        assert builder._match_path is False
        assert builder._match_whole_word is False
        assert builder._regex is False
        assert builder._sort_type == SortType.NAME_ASCENDING
        assert builder._max_results is None

    def test_keywords(self):
        """测试添加关键词"""
        builder = SearchBuilder()
        result = builder.keywords("test", "file")
        assert result is builder  # 链式调用
        assert builder._keywords == ["test", "file"]

    def test_add_filter(self):
        """测试添加过滤器"""
        builder = SearchBuilder()
        file_filter = FileFilter()
        result = builder.filter(file_filter)
        assert result is builder
        assert file_filter in builder._filters

    def test_match_case(self):
        """测试区分大小写"""
        builder = SearchBuilder()
        result = builder.match_case(True)
        assert result is builder
        assert builder._match_case is True

    def test_match_path(self):
        """测试匹配路径"""
        builder = SearchBuilder()
        result = builder.match_path(True)
        assert result is builder
        assert builder._match_path is True

    def test_match_whole_word(self):
        """测试全字匹配"""
        builder = SearchBuilder()
        result = builder.match_whole_word(True)
        assert result is builder
        assert builder._match_whole_word is True

    def test_use_regex(self):
        """测试使用正则表达式"""
        builder = SearchBuilder()
        result = builder.use_regex(True)
        assert result is builder
        assert builder._regex is True

    def test_sort_by(self):
        """测试排序"""
        builder = SearchBuilder()
        result = builder.sort_by(SortType.SIZE_DESCENDING)
        assert result is builder
        assert builder._sort_type == SortType.SIZE_DESCENDING

    def test_limit(self):
        """测试限制结果数量"""
        builder = SearchBuilder()
        result = builder.limit(100)
        assert result is builder
        assert builder._max_results == 100

    def test_build_query_string_keywords_only(self):
        """测试构建查询字符串（仅关键词）"""
        builder = SearchBuilder()
        builder.keywords("test", "file")
        query = builder.build_query_string()
        assert "test" in query
        assert "file" in query

    def test_build_query_string_with_filters(self):
        """测试构建查询字符串（包含过滤器）"""
        builder = SearchBuilder()
        builder.keywords("test")
        file_filter = FileFilter().with_extensions("txt")
        builder.filter(file_filter)
        query = builder.build_query_string()
        assert "test" in query
        assert "ext:txt" in query

    @patch("everytools.query.search.get_dll_loader")
    def test_execute(self, mock_loader):
        """测试执行搜索"""
        # 设置mock
        mock_dll = Mock()
        mock_loader.return_value.everything_dll = mock_dll
        mock_loader.return_value.check_error = Mock()
        mock_dll.Everything_QueryW.return_value = True
        mock_dll.Everything_GetNumResults.return_value = 0

        builder = SearchBuilder()
        builder.keywords("test")

        result = builder.execute()
        mock_dll.Everything_SetSearchW.assert_called()
        mock_dll.Everything_QueryW.assert_called_once_with(True)

        # 验证返回的是Search对象
        from everytools.query.search import Search

        assert isinstance(result, Search)
