#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
time_utils模块的单元测试
"""

import pytest
import struct
import datetime
from everytools.utils.time_utils import (
    filetime_to_datetime,
    filetime_to_timestamp,
    filetime_to_str,
    datetime_to_filetime,
    MAX_WINDOWS_TICKS,
    WINDOWS_TICKS,
    WINDOWS_TICKS_TO_POSIX_EPOCH,
)


class TestTimeUtils:
    def test_filetime_to_datetime(self):
        # 测试正常情况
        # 构造2020-01-01 00:00:00的FILETIME
        timestamp = datetime.datetime(2020, 1, 1, 0, 0, 0).timestamp()
        winticks = int(timestamp * WINDOWS_TICKS + WINDOWS_TICKS_TO_POSIX_EPOCH)
        filetime = struct.pack("<Q", winticks)

        dt = filetime_to_datetime(filetime)
        assert dt.year == 2020
        assert dt.month == 1
        assert dt.day == 1

        # 测试最大值（表示未知）
        filetime = struct.pack("<Q", MAX_WINDOWS_TICKS)
        assert filetime_to_datetime(filetime) is None

        # 测试小于WINDOWS_TICKS_TO_POSIX_EPOCH的值
        filetime = struct.pack("<Q", int(WINDOWS_TICKS_TO_POSIX_EPOCH) - 1)
        assert filetime_to_datetime(filetime) is None

    def test_filetime_to_timestamp(self):
        # 测试正常情况
        timestamp = 1577836800  # 2020-01-01 00:00:00
        winticks = int(timestamp * WINDOWS_TICKS + WINDOWS_TICKS_TO_POSIX_EPOCH)
        filetime = struct.pack("<Q", winticks)

        result = filetime_to_timestamp(filetime)
        assert abs(result - timestamp) < 0.001  # 允许小误差

        # 测试最大值（表示未知）
        filetime = struct.pack("<Q", MAX_WINDOWS_TICKS)
        assert filetime_to_timestamp(filetime) is None

    def test_filetime_to_str(self):
        # 测试正常情况
        timestamp = 1577836800  # 2020-01-01 00:00:00
        winticks = int(timestamp * WINDOWS_TICKS + WINDOWS_TICKS_TO_POSIX_EPOCH)
        filetime = struct.pack("<Q", winticks)

        # 注意：这里使用本地时区，所以具体的输出可能会根据测试环境的时区而变化
        result = filetime_to_str(filetime)
        assert isinstance(result, str)

        # 测试格式化字符串
        result = filetime_to_str(filetime, "%Y/%m/%d")
        assert "/" in result

        # 测试最大值（表示未知）
        filetime = struct.pack("<Q", MAX_WINDOWS_TICKS)
        assert filetime_to_str(filetime) is None

    def test_datetime_to_filetime(self):
        # 测试datetime对象
        dt = datetime.datetime(2020, 1, 1, 0, 0, 0)
        filetime = datetime_to_filetime(dt)
        unpacked = struct.unpack("<Q", filetime)[0]
        assert unpacked > WINDOWS_TICKS_TO_POSIX_EPOCH

        # 测试时间戳
        timestamp = 1577836800  # 2020-01-01 00:00:00
        filetime = datetime_to_filetime(timestamp)
        unpacked = struct.unpack("<Q", filetime)[0]
        assert unpacked > WINDOWS_TICKS_TO_POSIX_EPOCH

        # 测试None
        filetime = datetime_to_filetime(None)
        unpacked = struct.unpack("<Q", filetime)[0]
        assert unpacked == MAX_WINDOWS_TICKS
